part of './packets.dart';

class ConnectPacket extends MqttMessage {
  final String _protocolName = "MQTT";
  final int _rotocolVersion = 4;
  bool _cleanSession = true;
  final bool _willFlag = false;
  final MqttQos _willQos = MqttQos.atMostOnce;
  final bool _willRetain = false;
  bool _userNameFlag = false;
  bool _passwordFlag = false;
  // final int _ReservedBit = 0;
  int _keepalive = 0;
  String _clientIdentifier = "";
  String _userName = "";
  String _pwd = "";
  ConnectPacket() {
    head = MqttHeader().asType(MqttMessageType.CONNECT);
  }

  ConnectPacket withKeepalive(int t) {
    _keepalive = t;
    return this;
  }

  int getKeepalive() => _keepalive;

  ConnectPacket withAuth(String? userName, String? pwd) {
    if (userName != null) {
      _userNameFlag = userName.isNotEmpty;
      _userName = userName;
    } else {
      _userNameFlag = false;
    }

    if (pwd != null) {
      _passwordFlag = pwd.isNotEmpty;
      _pwd = pwd;
    } else {
      _passwordFlag = false;
    }
    return this;
  }

  ConnectPacket withClearSession(bool clear) {
    _cleanSession = clear;
    return this;
  }

  ConnectPacket withClientID(String clientID) {
    _clientIdentifier = clientID;
    return this;
  }

  @override
  Uint8List pack() {
    head.type = MqttMessageType.CONNECT;
    var buf = <int>[];
    buf.addAll(encodeString(_protocolName));
    buf.add(_rotocolVersion);
    buf.add(boolToByte(_cleanSession) << 1 |
        boolToByte(_willFlag) << 2 |
        _willQos.index << 3 |
        boolToByte(_willRetain) << 5 |
        boolToByte(_passwordFlag) << 6 |
        boolToByte(_userNameFlag) << 7);
    buf.addAll(encodeUint16(_keepalive));
    buf.addAll(encodeString(_clientIdentifier));
    if (_willFlag) {
      buf.addAll(encodeString("will"));
      buf.addAll(encodeString(""));
    }
    if (_userNameFlag) {
      buf.addAll(encodeString(_userName));
    }
    if (_passwordFlag) {
      buf.addAll(encodeString(_pwd));
    }
    head.remainingLength = buf.length;
    var buf2 = <int>[];
    buf2.addAll(head.pack());
    buf2.addAll(buf);
    return Uint8List.fromList(buf2);
  }
}
