part of './packets.dart';

class MqttHeader {
  // MqttHeader();
  MqttMessageType type = MqttMessageType.UNKNOW;
  bool dup = false;
  MqttQos qos = MqttQos.atMostOnce;
  bool retain = false;
  int remainingLength = 0;
  int _messageSize = 0;

  Uint8List pack() {
    var buf = <int>[];
    // xxxx)
    var x = type.index << 4 |
        boolToByte(dup) << 3 |
        qos.index << 1 |
        boolToByte(retain);
    buf.add(x);
    buf.addAll(getRemainingLengthBytes(remainingLength));
    return Uint8List.fromList(buf);
  }

  MqttHeader();
  MqttHeader.fromByteBuffer(MqttByteBuffer headerStream) {
    readFrom(headerStream);
  }

  MqttHeader asType(MqttMessageType messageType) {
    type = messageType;
    return this;
  }

  void readFrom(MqttByteBuffer b) {
    // final obj = MqttHeader._internal();
    if (b.length < 2) {
      b.reset();
      throw Exception('The supplied header is invalid. Header must be at '
          'least 2 bytes long.');
    }
    final firstHeaderByte = b.readByte();
    retain = ((firstHeaderByte & 1) == 1);
    qos = MqttQos.values.firstWhere(
        (item) => item.index == (firstHeaderByte & 6) >> 1,
        orElse: () => MqttQos.atMostOnce);
    dup = ((firstHeaderByte & 8) >> 3) == 1;
    type = MqttMessageType.values[((firstHeaderByte & 240) >> 4)];
    try {
      _messageSize = readRemainingLength(b);
    } on Exception {
      throw Exception(
          'The header being processed contained an invalid size byte pattern. '
          'Message size must take a most 4 bytes, and the last byte '
          'must have bit 8 set to 0.');
    } on Error {
      throw Exception(
          'The header being processed contained an invalid size byte pattern. '
          'Message size must take a most 4 bytes, and the last byte '
          'must have bit 8 set to 0.');
    }
  }

  @override
  String toString() => 'Header: MessageType = $type, Duplicate = $dup, '
      'Retain = $retain, Qos = $qos, Size = $_messageSize';
}
