part of './packets.dart';

class SubscribePacket extends MqttMessage {
  int _messageID = 0;
  final List<String> _topics = [];
  final List<MqttQos> _qoss = [];

  List<String> get topics {
    return _topics;
  }

  @override
  Uint8List pack() {
    head.type = MqttMessageType.SUBSCRIBE;
    var buf = <int>[];
    buf.addAll(encodeUint16(_messageID));
    for (var i = 0; i < _topics.length; i++) {
      buf.addAll(encodeString(_topics[i]));
      buf.add(_qoss[i].index);
    }
    head.remainingLength = buf.length;

    var buf2 = <int>[];
    buf2.addAll(head.pack());
    buf2.addAll(buf);
    return Uint8List.fromList(buf2);
  }

  void withMessageID(int val) {
    _messageID = val;
  }

  void add(String topic, MqttQos qos) {
    _topics.add(topic);
    _qoss.add(qos);
  }

  SubscribePacket.withTopic(int val, String topic) {
    withMessageID(val);
    add(topic, MqttQos.atLeastOnce);
  }

  SubscribePacket.withTopics(int val, List<String> topics) {
    withMessageID(val);
    for (var element in topics) {
      add(element, MqttQos.atLeastOnce);
    }
  }
  @override
  String toString() => 'SubPacket'
      '$_topics[$_messageID]';
}
