// Map<int, List<String>> msgIDs = Map<int, List<String>>();
import 'dart:convert';
import 'dart:typed_data';
import './mqtt_byte_buffer.dart';

int _currentMsgID = 0;

int boolToByte(bool val) {
  return val ? 1 : 0;
}

int getFreeID() {
  _currentMsgID++;
  if (_currentMsgID > 0xFFFE) {
    _currentMsgID = 1;
  }
  return _currentMsgID;
}

void resetFreeID() {
  _currentMsgID = 0;
}

Uint8List encodeString(String val) {
  return encodeBytes(Uint8List.fromList(utf8.encode(val)));
}

Uint8List encodeBytes(Uint8List stringToWrite) {
  final stringBytes = <int>[];
  stringBytes.addAll(encodeUint16(stringToWrite.length));
  stringBytes.addAll(stringToWrite);
  return Uint8List.fromList(stringBytes);
}

Uint8List encodeUint16(int val) {
  val = val & 0xFFFF;
  var stringBytes = Uint8List(2);
  ByteData.sublistView(stringBytes).setInt16(0, val);
  return stringBytes;
}

Uint8List getRemainingLengthBytes(int payloadCalc) {
  // List<int> lengthBytes = List();
  var buf = <int>[];
  // Generate a byte array based on the message size, splitting it up into
  // 7 bit chunks, with the 8th bit being used to indicate 'one more to come'
  do {
    var nextByteValue = payloadCalc % 128;
    payloadCalc = payloadCalc ~/ 128;
    if (payloadCalc > 0) {
      nextByteValue = nextByteValue | 0x80;
    }
    buf.add(nextByteValue);
  } while (payloadCalc > 0);
  return Uint8List.fromList(buf);
}

// /// Reads the length bytes of an MqttHeader from the supplied stream.
Uint8List readLengthBytes(MqttByteBuffer headerStream) {
  final lengthBytes = <int>[];
  // Read until we've got the entire size, or the 4 byte limit is reached
  int sizeByte;
  var byteCount = 0;
  do {
    sizeByte = headerStream.readByte();
    lengthBytes.add(sizeByte);
  } while (++byteCount <= 4 && (sizeByte & 0x80) == 0x80);
  return Uint8List.fromList(lengthBytes);
}

int calculateLength(Uint8List lengthBytes) {
  var remainingLength = 0;
  var multiplier = 1;

  for (final currentByte in lengthBytes) {
    remainingLength += (currentByte & 0x7f) * multiplier;
    multiplier *= 0x80;
  }
  return remainingLength;
}

readRemainingLength(MqttByteBuffer headerStream) {
  return calculateLength(readLengthBytes(headerStream));
}
